module Fractals(
        hilbert,
        join, split, copy
       )
        where


import LSystem


split :: Command -> [Command]
split Sit         =  []
split (Go x)      =  [Go x]
split (Turn x)    =  [Turn x]
split (GrabPen x) = [GrabPen x]
split (x :#: y)   =  split x ++ split y

join :: [Command] -> Command
join []      =  Sit
join (x:xs)  =  x :#: join xs

copy :: Int -> Command -> Command
copy 0 c = Sit
copy i c = c :#: copy (i-1) c

polygon :: Distance -> Int -> Command
polygon d i = copy i (Go d :#: Turn ((360.0/(fromIntegral i))))

optimise :: Command -> Command
optimise c = join (optRec (split c))

optRec :: [Command] -> [Command]
optRec []            = []
optRec [Sit]         = []
optRec [GrabPen x]   = [GrabPen x]
optRec [Go x]
          |x == 0    = []
          |otherwise = [Go x]
optRec [Turn x]
          |x == 0    = []
          |otherwise = [Turn x]
optRec ((Go x) : (Go y) : z)     = optRec ([Go (x+y)] ++ optRec z)
optRec ((Turn x) : (Turn y) : z) = optRec ([Turn (x+y)] ++ optRec z)
optRec (x:y)                     = optRec [x] ++ optRec y


-- Fractal

hilbert :: Int -> Command
hilbert x = optimise (l x)
  where
        l 0     = Sit
        l (x+1) = p :#: r x :#: f x :#: n :#: l x :#: f x :#: l x :#: n :#: f x :#: r x :#: p
        r 0     = Sit
        r (x+1) = n :#: l x :#: f x :#: p :#: r x :#: f x :#: r x :#: p :#: f x :#: l x :#: n
	f x     = Go 10
        n       = Turn 90
        p       = Turn (-90)
